# Правила проекта MCP Microservice

## 1. Стандарты

### 1.1. Стандарты кода
- Каждая команда в отдельном файле
- Строгая типизация всех параметров и возвращаемых значений
- Асинхронное выполнение команд
- Единый формат ответов через JSON-RPC
- Стандартизированная обработка ошибок

### 1.2. Стандарты именования
- snake_case для файлов и функций
- PascalCase для классов
- UPPER_CASE для констант и файлов документации
- Префикс `_` для protected методов
- Префикс `__` для private методов

### 1.3. Стандарты структуры файлов
```python
# Порядок секций в файлах команд:
1. Docstring модуля
2. Импорты (стандартные → сторонние → проекта)
3. Константы и типы
4. Класс результата
5. Вспомогательные функции
6. Команда
```

### 1.4. Стандарты документации
- **ОБЯЗАТЕЛЬНО** вести документацию на двух языках:
  - Русский (директория `/docs/RU/`)
  - English (директория `/docs/EN/`)
- **Документация на русском ДОЛЖНА быть точной копией английской версии:**
  - Идентичная структура файлов
  - Идентичная структура разделов
  - Идентичные примеры кода
  - Одинаковое форматирование
  - Одинаковые диаграммы и схемы
- Запрещено:
  - Добавлять документацию только на одном языке
  - Иметь различия в структуре или содержании между версиями
  - Обновлять одну версию без обновления другой
- При внесении изменений:
  - Изменения вносятся одновременно в обе версии
  - Проводится проверка соответствия версий
  - Коммит должен содержать изменения в обеих версиях

## 2. Чеклисты

### 2.1. Добавление новой команды
- [ ] Создать файл `{command_name}_command.py`
- [ ] Определить класс результата
- [ ] Реализовать методы `to_dict()` и `get_schema()`
- [ ] Реализовать команду
- [ ] Добавить тесты
- [ ] Создать документацию

### 2.2. Тестирование команды
- [ ] Тесты успешных сценариев
- [ ] Тесты ошибок
- [ ] Проверка валидации параметров
- [ ] Проверка сериализации результата
- [ ] Проверка генерации схемы

### 2.3. Проверка документации
- [ ] Docstring модуля
- [ ] Docstring класса результата
- [ ] Docstring команды
- [ ] Примеры использования
- [ ] Описание параметров
- [ ] Примеры JSON-RPC запросов/ответов
- [ ] **Проверка соответствия русской и английской версий:**
  - [ ] Структура файлов и разделов
  - [ ] Примеры кода
  - [ ] Форматирование
  - [ ] Диаграммы и схемы
  - [ ] Полнота перевода

### 2.4. Code Review
- [ ] Соответствие стандартам именования
- [ ] Корректность типизации
- [ ] Наличие тестов
- [ ] Полнота документации
- [ ] Обработка ошибок
- [ ] **Проверка двуязычной документации:**
  - [ ] Наличие обеих версий
  - [ ] Идентичность структуры
  - [ ] Синхронность изменений

## 3. Документация

### 3.1. Структура документации
```
docs/
├── EN/                 # English documentation (mandatory)
│   ├── commands/       # Command documentation
│   │   └── {command_name}_command.md
│   ├── PROJECT_RULES.md
│   ├── NAMING_STANDARDS.md
│   └── api/           # API documentation
└── RU/                # Русская документация (обязательно)
    ├── commands/      # Документация команд
    │   └── {command_name}_command.md
    ├── PROJECT_RULES.md
    ├── NAMING_STANDARDS.md
    └── api/           # Документация API
```

### 3.2. Обязательные разделы в документации команды
1. Описание (на обоих языках)
2. Результат (с примером кода)
3. Команда (с примером кода)
4. Особенности реализации
5. Примеры использования
   - Python
   - HTTP REST
   - JSON-RPC

### 3.3. Требования к документации
- Актуальность примеров кода
- Описание всех параметров
- Описание возможных ошибок
- Примеры успешных и неуспешных сценариев
- Схемы валидации данных
- **Синхронизация русской и английской версий**
- **Одинаковая структура в обеих версиях**
- **Идентичные примеры кода в обеих версиях**

### 3.4. Форматирование документации
- Markdown для всех файлов
- Mermaid для диаграмм
- Форматированные блоки кода с указанием языка
- Таблицы для структурированных данных

## 4. Жизненный цикл

### 4.1. Разработка
1. Создание файла команды
2. Реализация результата
3. Реализация команды
4. Написание тестов
5. Создание документации

### 4.2. Тестирование
1. Модульные тесты
2. Интеграционные тесты
3. Проверка документации
4. Проверка схемы API

### 4.3. Развертывание
1. Проверка зависимостей
2. Обновление схемы API
3. Миграция данных (если требуется)
4. Обновление документации

### 4.4. Поддержка
1. Мониторинг выполнения
2. Анализ ошибок
3. Обновление при необходимости
4. Поддержка обратной совместимости 