# Метаданные команд

Система метаданных команд предоставляет единый механизм для получения структурированной информации о командах, их параметрах, результатах и примерах использования.

## Содержание

1. [Обзор](#обзор)
2. [Структура метаданных](#структура-метаданных)
3. [Получение метаданных](#получение-метаданных)
4. [Генерация примеров](#генерация-примеров)
5. [Использование в команде help](#использование-в-команде-help)
6. [Расширение метаданных](#расширение-метаданных)

## Обзор

Метаданные команд позволяют унифицированно получать информацию о доступных командах микросервиса для:

- Автодокументирования API
- Генерации клиентских библиотек
- Построения интерактивных интерфейсов
- Улучшения пользовательского опыта при использовании API

## Структура метаданных

Метаданные команды представляют собой словарь со следующими ключами:

```python
{
    "name": "имя_команды",
    "summary": "Краткое описание команды",
    "description": "Полное описание команды...",
    "params": {
        "имя_параметра": {
            "name": "имя_параметра",
            "required": True/False,
            "type": "тип_параметра",
            "default": "значение_по_умолчанию"  # если есть
        },
        # другие параметры
    },
    "examples": [
        {
            "command": "имя_команды",
            "params": {"параметр": "значение"},
            "description": "Описание примера"
        },
        # другие примеры
    ],
    "schema": { /* JSON-схема команды */ },
    "result_schema": { /* JSON-схема результата */ },
    "result_class": "ИмяКлассаРезультата"
}
```

## Получение метаданных

Базовый класс `Command` предоставляет два метода для получения метаданных:

### Получение метаданных одной команды

```python
# Через класс команды
metadata = SomeCommand.get_metadata()

# Через реестр команд
metadata = registry.get_command_metadata("command_name")
```

### Получение метаданных всех команд

```python
# Получение метаданных всех команд
all_metadata = registry.get_all_metadata()
```

## Генерация примеров

Система автоматически генерирует примеры использования команд на основе параметров:

1. Пример без параметров (если все параметры опциональны)
2. Пример с обязательными параметрами
3. Пример со всеми параметрами (обязательными и опциональными)

Значения для примеров генерируются на основе типа параметра:
- `str` → `"example_param_name"`
- `int` → `123`
- `float` → `123.45`
- `bool` → `True`

## Использование в команде help

Команда `help` использует метаданные для предоставления информации о доступных командах:

```bash
# Получение списка всех команд
{"command": "help"}

# Получение информации о конкретной команде
{"command": "help", "params": {"cmdname": "command_name"}}
```

Результат команды help содержит:

1. При запросе без параметров:
   - Информацию о микросервисе
   - Список всех доступных команд с кратким описанием
   - Примеры использования команды help

2. При запросе с параметром cmdname:
   - Полное описание команды
   - Список параметров с типами и обязательностью
   - Примеры использования команды

## Расширение метаданных

Для расширения метаданных в конкретных командах можно переопределить метод `get_metadata()`:

```python
@classmethod
def get_metadata(cls) -> Dict[str, Any]:
    # Получение базовых метаданных
    metadata = super().get_metadata()
    
    # Расширение метаданных
    metadata["category"] = "system"
    metadata["permissions"] = ["admin"]
    
    return metadata
```

### Кастомные примеры

Для предоставления специальных примеров можно переопределить метод `_generate_examples()`:

```python
@classmethod
def _generate_examples(cls, params: Dict[str, Dict[str, Any]]) -> List[Dict[str, Any]]:
    # Получение стандартных примеров
    examples = super()._generate_examples(params)
    
    # Добавление кастомного примера
    examples.append({
        "command": cls.name,
        "params": {"special_param": "special_value"},
        "description": "Специальный пример использования"
    })
    
    return examples
``` 